from Language.TurkishLanguage import TurkishLanguage
from Syllibification.Syllable import Syllable


class SyllableList:
    __syllables: list

    def __init__(self, word: str):
        """
         A constructor of SyllableList class which takes a String word as an input. First it creates a syllable
         list and a sbSyllable. Then it loops i times, where i ranges from 0 to length of given word, first
         it gets the ith character of given word and checks whether it is a vowel and the last character of the word.

         If it is a vowel it appends it to the sbSyllable and if it is the last vowel it also appends the next character
         to the sbSyllable. Then, it adds the sbSyllable to the syllables list.

         If it is not a vowel, and the sbSyllable's length is 1 also the previous character is a consonant it gets the
         last item of syllables list since there cannot be a Turkish word which starts with two consonants. However, if
         it is two last characters of word, then it adds it to the syllablelist. At the end, it updates the syllables
         list.

        PARAMETERS
        ----------
        word : str
            String input.
        """
        self.__syllables = []
        sbSyllable = ""
        i = 0
        while i < len(word):
            c = word[i]
            isVowel = TurkishLanguage.isVowel(c)
            isLastChar = (i == len(word) - 1)
            if isVowel:
                sbSyllable += c
                # If it is the last vowel.
                if i == len(word) - 2:
                    sbSyllable += word[i + 1]
                    i = i + 1
                self.__syllables.append(Syllable(sbSyllable))
                sbSyllable = ""
            else:
                # A syllable should not start with two consonants.
                tempSyl = sbSyllable
                if len(tempSyl) == 1:
                    # The previous character was also a consonant.
                    if not TurkishLanguage.isVowel(tempSyl[0]):
                        if len(self.__syllables) == 0:
                            sbSyllable += c
                            i = i + 1
                            continue
                        lastPos = len(self.__syllables) - 1
                        string = self.__syllables[lastPos].getText()
                        string = string + tempSyl
                        if isLastChar:
                            # If the last char is also a consonant, add it to latest syllable. Ex: 'park'.
                            string = string + c
                        # Update previous syllable.
                        self.__syllables[lastPos] = Syllable(string)
                        sbSyllable = ""
                sbSyllable += c
            i = i + 1

    def getSyllables(self) -> list:
        """
        The getSyllables method creates a new list syllables and loops through the globally defined syllables
        list and adds each item to the newly created syllables list.

        RETURNS
        -------
        list
            list syllables.
        """
        syllables = []
        for syllable in self.__syllables:
            syllables.append(syllable.getText())
        return syllables



from Dictionary.Trie.TrieNode import TrieNode
from Dictionary.Word import Word
from Dictionary.TxtWord import TxtWord


class Trie:

    __rootNode: TrieNode

    def __init__(self):
        """
        A constructor of Trie class which creates a new TrieNode as rootNode.
        """
        self.__rootNode = TrieNode()

    def addWord(self, word: str, root: Word):
        """
        The addWord method which takes a String word and a Word root as inputs and adds given word and root to the
        rootNode.

        PARAMETERS
        ----------
        word : str
            String input.
        root : Word
            Word input.
        """
        self.__rootNode.addWord(word, root)

    def getWordsWithPrefix(self, surfaceForm: str) -> set:
        """
        The getWordsWithPrefix method which takes a String surfaceForm as an input. First it creates a TrieNode current
        and assigns the rootNode to it, then it creates a new set words. It loops i times where i ranges from 0 to
        length of surfaceForm and assigns current's child that corresponds to the surfaceForm's char at index i and
        assigns it as TrieNode current. If current is not None, it adds all words of current to the words set.

        PARAMETERS
        ----------
        surfaceForm : str
            String input.

        RETURNS
        -------
        set
            words set.
        """
        current = self.__rootNode
        words = set()
        for i in range(len(surfaceForm)):
            current = current.getChild(surfaceForm[i])
            if current is not None:
                words.update(current.getWords())
            else:
                break
        return words

    def getCompundWordStartingWith(self, _hash: str) -> TxtWord:
        """
        The getCompoundWordStartingWith method takes a String hash. First it creates a TrieNode current and assigns
        the rootNode to it. Then it loops i times where i ranges from 0 to length of given hash and assigns current's
        child that corresponds to the hash's char at index i and assigns it as current. If current is None, it returns
        null.

        If current is not None, it loops through the words of current TrieNode and if it is a Portmanteau word, it
        directly returns the word.

        PARAMETERS
        ----------
        _hash : str
            String input.

        RETURNS
        -------
        TxtWord
            None if TrieNode is None, otherwise portmanteau word.
        """
        current = self.__rootNode
        for i in range(len(_hash)):
            current = current.getChild(_hash[i])
            if current is None:
                return None
        if current is not None:
            for word in current.getWords():
                if word.isPortmanteau():
                    return word
        return None



class Syllable:

    __syllable: str

    def __init__(self, syllable: str):
        """
        A constructor of Syllable class which takes a String as an input and initializes syllable variable with given input.

        PARAMETERS
        ----------
        syllable : str
            String input.
        """
        self.__syllable = syllable

    def getText(self) -> str:
        """
        Getter for the syllable variable.

        RETURNS
        -------
        str
            The syllable variable.
        """
        return self.__syllable



from Dictionary.Word import Word
from Math.Vector import Vector


class VectorizedWord(Word):

    __vector: Vector

    def __init__(self, name: str, vector: Vector):
        """
        A constructor of VectorizedWord class which takes a String and a Vector as inputs and calls its
        super class Word with name and also initializes vector variable with given input.

        PARAMETERS
        ----------
        name : str
            String input.
        vector : Vector
            Vector type input.
        """
        super().__init__(name)
        self.__vector = vector

    def getVector(self) -> Vector:
        """
        Getter for the vector variable.

        RETURNS
        -------
        Vector
            the vector variable.
        """
        return self.__vector



from Language.Language import Language


class TurkishLanguage(Language):
    VOWELS = "aeıioöuüâî"
    BACK_VOWELS = "aıouâ"
    FRONT_VOWELS = "eiöüî"
    BACK_ROUNDED_VOWELS = "ou"
    BACK_UNROUNDED_VOWELS = "aıâ"
    FRONT_ROUNDED_VOWELS = "öü"
    FRONT_UNROUNDED_VOWELS = "eiî"
    CONSONANT_DROPS = "nsy"
    CONSONANTS = "bcçdfgğhjklmnprsştvyzxqw"
    SERT_SESSIZ = "çfhkpsşt"
    LOWERCASE_LETTERS = "abcçdefgğhıijklmnoöprsştuüvyz"
    UPPERCASE_LETTERS = "ABCÇDEFGĞHIİJKLMNOÖPRSŞTUÜVYZ"
    LETTERS = LOWERCASE_LETTERS + UPPERCASE_LETTERS

    @staticmethod
    def isVowel(ch: str) -> bool:
        """
        The isVowel method takes a character as an input and returns true if given character is a vowel.

        PARAMETERS
        ----------
        ch : chr
            Character input to check.

        RETURNS
        -------
        bool
            true if given character is a vowel.
        """
        return ch in TurkishLanguage.VOWELS

    @staticmethod
    def isBackVowel(ch: str) -> bool:
        """
        The isBackVowel method takes a character as an input and returns true if given character is a back vowel.

        PARAMETERS
        ----------
        ch : chr
            Character input to check.

        RETURNS
        -------
        bool
            true if given character is a back vowel.
        """
        return ch in TurkishLanguage.BACK_VOWELS

    @staticmethod
    def isFrontVowel(ch: str) -> bool:
        """
        The isFrontVowel method takes a character as an input and returns true if given character is a front vowel.

        PARAMETERS
        ----------
        ch : chr
            Character input to check.

        RETURNS
        -------
        bool
            true if given character is a front vowel.
        """
        return ch in TurkishLanguage.FRONT_VOWELS

    @staticmethod
    def isBackRoundedVowel(ch: str) -> bool:
        """
        The isBackRoundedVowel method takes a character as an input and returns true if given character is a back
        rounded vowel.

        PARAMETERS
        ----------
        ch : chr
            Character input to check.

        RETURNS
        -------
        bool
            true if given character is a back rounded vowel.
        """
        return ch in TurkishLanguage.BACK_ROUNDED_VOWELS

    @staticmethod
    def isFrontRoundedVowel(ch: str) -> bool:
        """
        The isFrontRoundedVowel method takes a character as an input and returns true if given character is a front
        rounded vowel.

        PARAMETERS
        ----------
        ch : chr
            Character input to check.

        RETURNS
        -------
        bool
            true if given character is a front rounded vowel.
        """
        return ch in TurkishLanguage.FRONT_ROUNDED_VOWELS

    @staticmethod
    def isBackUnroundedVowel(ch: str) -> bool:
        """
        The isBackUnroundedVowel method takes a character as an input and returns true if given character is a back
        unrounded vowel.

        PARAMETERS
        ----------
        ch : chr
            Character input to check.

        RETURNS
        -------
        bool
            true if given character is a back unrounded vowel.
        """
        return ch in TurkishLanguage.BACK_UNROUNDED_VOWELS

    @staticmethod
    def isFrontUnroundedVowel(ch: str) -> bool:
        """
        The isFrontUnroundedVowel method takes a character as an input and returns true if given character is a front
        unrounded vowel.

        PARAMETERS
        ----------
        ch : chr
            Character input to check.

        RETURNS
        -------
        bool
            true if given character is a front unrounded vowel.
        """
        return ch in TurkishLanguage.FRONT_UNROUNDED_VOWELS

    @staticmethod
    def isConsonantDrop(ch: str) -> bool:
        """
        The isConsonantDrop method takes a character as an input and returns true if given character is a dropping
        consonant

        PARAMETERS
        ----------
        ch : chr
            Character input to check.

        RETURNS
        -------
        bool
            true if given character is a dropping consonant.
        """
        return ch in TurkishLanguage.CONSONANT_DROPS

    @staticmethod
    def isConsonant(ch: str) -> bool:
        """
        The isConsonant method takes a character as an input and returns true if given character is a consonant

        PARAMETERS
        ----------
        ch : chr
            Character input to check.

        RETURNS
        -------
        bool
            true if given character is a consonant.
        """
        return ch in TurkishLanguage.CONSONANTS

    @staticmethod
    def isSertSessiz(ch: str) -> bool:
        """
        The isSertSessiz method takes a character as an input and returns true if given character is a sert sessiz

        PARAMETERS
        ----------
        ch : chr
            Character input to check.

        RETURNS
        -------
        bool
            true if given character is a sert sessiz.
        """
        return ch in TurkishLanguage.SERT_SESSIZ



from enum import Enum, auto


class Pos(Enum):
    """
    Parts of speech.
    """

    """
    Adjective.
    """
    ADJECTIVE = auto()
    """
    Noun.
    """
    NOUN = auto()
    """
    Verb.
    """
    VERB = auto()
    """
    Adverb.
    """
    ADVERB = auto()
    """
    Conjunction.
    """
    CONJUNCTION = auto()
    """
    Interjection.
    """
    INTERJECTION = auto()
    """
    Preposition.
    """
    PREPOSITION = auto()
    """
    Pronoun.
    """
    PRONOUN = auto()



from setuptools import setup

setup(
    name='NlpToolkit-Dictionary',
    version='1.0.20',
    packages=['Language', 'Dictionary', 'Dictionary.Trie', 'Syllibification'],
    url='https://github.com/olcaytaner/Dictionary-Py',
    license='',
    author='olcaytaner',
    author_email='olcaytaner@isikun.edu.tr',
    description='Simple Dictionary Processing',
    install_requires=['NlpToolkit-Math']
)
